\ MyCo \ is using the 4e4th Release0.34b
\ Version 0.91 - Possibility to edit pages and programs using only S2 and S1.
\ started: 2014_08_10 now 2015_04_04


\ Achtung: 
\ Zweistellige Ziffern sind Kommandos in MYCO !!
\ Willst du Zahlen eingeben, müssen Nullen davor sein.
\ Beispiel:
\ 59 lädt Register A in das PWM-Register.
\ 0059 legt den Zahlenwert 59 auf den Datenstack.


\ Issues:
\ WIPE strikes after EDIT ;        fixed 24.02.2015
\ ugly EDIT; made a better one;    12.10.2014
\ S1 and S2 input faild sometimes; fixed 12.10.2014 
\ STEP does wrong dumping;         fixed 08.10.2014


\ +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
\
\ Start of the Program  MyCo, programmed in Forth
\ This is based on Burkard Kainka's TPS; Juergen Pintaske
\ translated the German Manual into English and extended it.

\ This program in Forth simulates the same very small processor, that was 
\ running in the Franzis Kit and Conrad TPS Kity which is planned to help 
\ Learning Programming;
\ and we use it here as an example of how to program the TI MSP430 Controller. 
\ Owners of this Franzis or Conrad kit can just add a 
\ programmed MSP430G2553 and a reset resistor and 
\ replace the Holtek Controller.
\ And all three options can then be used to Learn Programming:
\   First, use the original TPS, running in a Holtek Processor, 
\      and using the Hardware available from Franzis or Conrad. 
\   Second, use the existing Hardware, take the Holtek chip out 
\      and add a little board with the MSP430 Controller on it. 
\      Just jumper the necessary wires to the pre-programmed MSP430G2553.
\   Third, get a pre-programmed MSP430 controller, add in addition to Reset RC
\      the 5 resistor and LED combinations and the 3 switches to 430 board,
\      and run it completely in the MSP430, but in addition now with a serial 
\      terminal control, showing on screen what actually happens 
\      internally in all of the simulated processor registers. 
\
\ This all looks like an overkill, but the target is here:  
\ Learn Programming in an Interactive Language called Forth 
\ -  and so this is actually an Example Application. 

\ A Note:  when using the Serial Terminal Function:
\ Use any hex numbers only with preceding zeros. 
\ Any hex XX without a proceeding 0 is interpreted by MyCo as a command. 
\ Example: 
\ 71 is the command A<=A+1, 0071 is interpreted as a number. 

\ Defining the I/O for this application

\ Device Pinout of the  MSP430G2553 20-Pin PDIP Controller:
\
\                     (TOP VIEW) 
\ VCC----+3.3V--------[01   20]--Ground-----------VSS 
\ (LED1 LP) AD1 P1.0--[02   19]--P2.6 OUT2 – 470R  + LED to GND
\ RXD-----------P1.1--[03   18]--P2.7 OUT3 – 470R  + LED to GND
\ TXD-----------P1.2--[04   17]--test
\ S2 to GND     P1.3--[05   16]--RST      S1-LP            to GND
\ ..........AD2 P1.4--[06   15]--P1.7     S1               to GND
\ _-_-_-_-_-FRQ P1.5--[07   14]--P1.6 PWM  – 470R  + LED to GND
\ ----------IN0 P2.0--[08   13]--P2.5 OUT1 – 470R  + LED to GND
\ ----------IN1 P2.1--[09   12]--P2.4 OUT0 – 470R  + LED to GND
\ ----------IN2 P2.2--[10   11]--P2.3 IN3 
\ LP = TI Launchpad
\
\ P1.0 (2)  used as Input, later on as Analog Input 1
\ P1.1 (3)  RX and 
\ P1.2 (4)  TX are used as the serial interface to the PC, mostly via USBtoTTL
\ P1.3 (5)  used as S2, as on the TI Launchpad, 
\           the internal resistor is enabled
\ P1.4 (6)  used as Input, later on as Analog Input 2
\ P1.5 (7)  used as Output, later on to output a frequency with defined length
\ P1.6 (14) Output, later on as Pulse Width Modulation output
\           - quasi D/A Output
\             resistor about 470 Ohms and LED to Ground
\ P1.7 (15) Input S1, internal resistor enabled

\ P2.0 (8)  Input 0  internal resistor enabled, so open input means HIGH
\ P2.1 (9)  Input 1  internal resistor enabled, so open input means HIGH
\ P2.2 (10) Input 2  internal resistor enabled, so open input means HIGH
\ P2.3 (11) Input 3  internal resistor enabled, so open input means HIGH

\ P2.4 (12) Output 0  resistor about 470 Ohms and LED to Ground
\ P2.5 (13) Output 1  resistor about 470 Ohms and LED to Ground
\ P2.6 (19) Output 2  resistor about 470 Ohms and LED to Ground
\ P2.7 (18) Output 3  resistor about 470 Ohms and LED to Ground

\ One of the important issues to be solved here was to use the exising 
\ FLASH memory of the MSP430 as an EEPROM functionality needed for 
\ local programming;
\ this is an internal issue for later and can be skipped for now
\ The original Holtek Controller has 128 Bytes of EEPROM available used there.

\ 'EEROM' simulaton in flash memory  -- see the chapter further down


\ The Programming Model of this Minimum Processor is simulated in Forth.
\ Remember: 
\ For simplicity this processor is used as 4 bit, everything is in nibbles.
\ This is about the same processor as in Burkhard Kainka's original design:
\ PC Program Counter - limited to address memory locations 0 - 127,  
\   00 to 7F in HEX
\ PG Page register 4 bit, combined with the 4 bit jump address achieves
\   256($100)range
\ DL Delay register, a delay from 1ms to 60 sec can be programmed.
\ SK Skip register, indicates a possible skip over instructions based on
\    comparisons
\    There are no conditional branches - just these Conditional Skips for
\    simplicity
\    Easy for beginners as the program code is relocatable 
\   (address independent)

\ The main processor registers and other registers:
\ IN  4 bit input, when used reflects the status of the 4 Input lines
\ A   4 bit A register, somehow the central register, 
\     the ALU works with A register
\ B   4 bit B register
\ C   4 bit C register
\ D   4 bit D register
\ A1  4 bit Analog Input Channel 1
\ A2  4 bit Analog Input Channel 2
\ PW  4 bit Pulse Width Modulated Output, 
\     can be used filtered as quasi analog output 
\ OU  4 bit output register, 
\     4 Output Pins will reflect the contents, LEDS display it

\ There are two options to program the program for this simulated processor:
\   via the switches (and needs setting the IN bits for page selection), 
\   or controled completely via the Serial Interface - mostly to a PC, 
\   after entering q,  so the more complex parts of this program are used to
\   show the processor status on the monitor. 
\   e.g. Display all of the variables in this processor, 
\   here formatted differently:

\  PC   PG DL SK    IN  A  B  C  D  A1 A2 PW  OU
\  x x  x  x  x     x   x  x  x  x  x  x  x   x      

\ The next part of the program code is related to showing how the FLASH Memory 
\ is used as simulated EEPROM, so skip over this for now.


\ *** Let us start with an empty user flash ***

HEX   \ Just set for terminal IO and display (the options are HEX  and DECIMAL)

\ membot 0000 + constant seg0 ; this did not work, see issues.
\ membot 0200 + constant seg1 

\ PAD constant CACHE  \ PAD is already there in forth, so we use it. 
\                     \  Commented out here
1000 constant EE0    \ start address of info-d
1040 constant EE1    \ start address of info-c
0040 constant SEGSIZ \ segment size
0010 constant BLKSIZ \ bytes in block (alias "page" of MyCo)

variable SEG  \ holding segment number in use
variable BLK  \ holding block number 
variable UPD  \ this variable shows if the cache has been updated

\ As we are simulating an existing system, 
\ we use the same TPS/MyCo example programs 
\ 128 bytes of example programs = 8 pages -- page0 .. page7
HEX 
IHERE 
\ ( There is a seperate file showing the same code explained. ) 
\ Page0   
64 ic, 51 ic, 4E ic, 80 ic, 
C3 ic, 98 ic, 82 ic, 95 ic, 
4D ic, 80 ic, C3 ic, 9E ic, 
82 ic, 9A ic, 4B ic, 81 ic, 
\ Page1 
C3 ic, 94 ic, 83 ic, 90 ic, 
47 ic, 81 ic, C3 ic, 9A ic, 
83 ic, 94 ic, 43 ic, 82 ic, 
C3 ic, 90 ic, 84 ic, 90 ic, 
\ Page2 
11 ic, 28 ic, 18 ic, 28 ic, 
34 ic, 71 ic, 54 ic, 59 ic, 
26 ic, 34 ic, 69 ic, 54 ic, 
59 ic, 26 ic, 34 ic, FF ic, 
\ Page3 
54 ic, CE ic, 71 ic, 33 ic, 
22 ic, CC ic, 32 ic, 40 ic, 
22 ic, 71 ic, 54 ic, CE ic, 
34 ic, 39 ic, FF ic, FF ic, 
\ Page4 
86 ic, D0 ic, 40 ic, 71 ic, 
54 ic, 23 ic, CD ic, 34 ic, 
D8 ic, 40 ic, 54 ic, 3B ic, 
FF ic, FF ic, FF ic, FF ic, 
\ Page5 
4F ic, 93 ic, 45 ic, 53 ic, 
19 ic, 11 ic, 21 ic, 19 ic, 
11 ic, 21 ic, 19 ic, 11 ic, 
20 ic, B4 ic, 10 ic, E0 ic, 
\ Page6 
23 ic, CE ic, 32 ic, 23 ic, 
CC ic, 31 ic, E0 ic, FF ic, 
23 ic, CF ic, 32 ic, 23 ic, 
CD ic, 31 ic, E0 ic, FF ic, 
\ Page7 
CC ic, 31 ic, 40 ic, 54 ic, 
23 ic, CE ic, 32 ic, CF ic, 
E0 ic, CC ic, 33 ic, 71 ic, 
23 ic, CC ic, 31 ic, 3C ic,
ihere 
constant EXEND 
constant EX0 


\ Now the real activity starts: The default programs above are copied into the 
\ 128 bytes of Program Area available for programming and will show some demos,
\ using this code after a Reset. 

HEX
: PRG0  \ --   \ copy the default programs to the 'eeprom' area. 
               \ This MCU has no EEPROM, so it is simulated in flash. 
  ee0 segsiz flerase  
  ee1 segsiz flerase
  ex0 ee0 exend ex0 - d->i  ;  


\ The memory of the MSP430 will be rather fully used. 
\ So, to save Program Space, the 4 Bit Nibbles are packaged into 4 nibbles
\ to be one Variable of 16 bits.
\ This will save on RAM used, and is as well a programming exercise.



\ Juggling nibbles    mk0 
\ (this mkx marks shows how extensively the code has been tested)
HEX 
: 0! \ n adr -- \ store nibble 0 to value at address 
  >r r@ @ FFF0 and swap F and + r> ! ; 

: 1! \ n adr -- \ store nibble 1 to value at address 
  >r r@ @ FF0F and swap F and 4 lshift + r> ! ; 

: 2! \ n adr -- \ store nibble 2 to value at address 
  >r r@ @ F0FF and swap F and 8 lshift + r> ! ; 

: 3! \ n adr -- \ store nibble 3 to value at address 
  >r r@ @ 0FFF and swap F and C lshift + r> ! ; 

: 0@ \ adr - n \ fetch nibble 0 of value at address 
  @ 000F and ; 

: 1@ \ adr - n \ fetch nibble 1 of value at address 
  @ 00F0 and 0004 rshift ; 

: 2@ \ adr - n \ fetch nibble 2 of value at address 
  @ 0F00 and 0008 rshift ; 

: 3@ \ adr - n \ fetch nibble 3 of value at address 
  @ F000 and 000C rshift ; 


\ As a first step, we have to define and program the relevant IO Bits 
\ of the chip as Input (with enabling internal pull-up resistor)
\ or as Output.
\ In this part of the program you can find the relevant memory mapped 
\ addresses to 
\   Set the IO functionality, 
\   Read Input Data, 
\   Write Output Data
\   Enable the internal resistors of the processor on inputs
\ Some of the internal MSP430 functions are rather complex, so here the 
\ IO Bits are used as simple Input / Output only, additional Forth Code later

HEX 
: INITIO \ -- \ I/O initialisation of the MyCo ports P1 and P2  mk0
\         mask           adr            op 
\       76543210 
[ bin ] 10011001 [ hex ] 0022 ( P1DIR ) cclr \ P1 INs 
[ bin ] 01100110 [ hex ] 0022 ( P1DIR ) cset \ P1 OUTs 
[ bin ] 10001000 [ hex ] 0024 ( P1IES ) cset \ falling edge detect
[ bin ] 00100000 [ hex ] 0026 ( P1SEL ) cset \ P1.5 sec func TA0.0 (FRQ) 
[ bin ] 10001000 [ hex ] 0027 ( P1REN ) cset \ pull-ups selected 
[ bin ] 10000000 [ hex ] P1             cset \ P1.7 pull-ups enabled 

[ bin ] 00001111 [ hex ] 002A ( P2DIR ) cclr \ P2 INs 
[ bin ] 11110000 [ hex ] 002A ( P2DIR ) cset \ P2 OUTs 
[ bin ] 00001111 [ hex ] 002F ( P2REN ) cset \ P2 pull-ups selected 
[ bin ] 00001111 [ hex ] P2             cset \ P2 pull-ups enabled 
[ bin ] 11000000 [ hex ] 002E ( P2SEL ) cclr \ clear these bits to I/O 
;    \ end of IO Setting Definition Word

INITIO     \ now execute this just defined word INITIO to get IO ready

HEX \ Set to HEX for some basic peripheral operations, 
\                rather than using DECIMAL IO

: OUT! \ n -- \ write to output pins (nibble, 0004 LEDs) 
              \ do NOT use the c! word - this would reset the pull-ups! 
  F0 p2 cclr 000F and 0004 lshift p2 cset ; 

023 constant P1IFG    \ define a constant P1IFG

: S1?  080 P1IFG cget ;   \ -- f   \ set on edge event.

: S1-  080 P1IFG cclr ;   \ --   \ reset flag

: S2?  008 P1IFG cget ;   \ -- f   \ set on edge event.

: S2-  008 P1IFG cclr ;   \ --   \ reset flag

: IN@ \ -- n \ read digital input pins (nibble ) 
  P2 1- c@ 000F and ; 


\ Now with the IO got ready we are ready to define the simulated processor;  
\ As explained before, the 4 bit Registers are packed into 16 bit variables 

\  SGPC PDSI ABCD 12WO
\  xxxx xxxx xxxx xxxx

HEX    \ use HEXADECIMAL for Input and Output

variable SGPC    \ Segment(4 bit), PageCounter(4 bit) and PC (2x 4 bit)

variable RTRN    \ ReTuRN address (copy of the SGPC before call happens) 

variable PDSI    \ Page_Preset,  Delay,  Skip,  Input 
 
variable ABCD    \ registers  A  B       C      D 

variable 12WO    \          ad1, ad2,    pWm,   out 

\ LED display operations

: OUT>  \ -- n   \ pull the OUT nibble onto stack
  12wo 0@ ;
: >OUT  \ n --   \ push n to the OUT nibble
  12wo 0! ;
: OUT.  \ n --   \ display n at OUT-LEDs, copy n to OUT-register 
  dup out! >out ; 


: ?  \ adr --   \ define a Word to show the byte ( 8 bits) at an address
  c@ . ; 
: ?? \ adr --   \ define a Word to show a cell   (16 bits) at address unsigned
  @ u. ; 

\ Now the next step is to define the instructions 
\ that this little processor can execute

\ Define The Processor Instructions:

\ The first group of Instructions covers 
\ 0n – where n has the values of 0 to F
\      Tones are output by the MSP430G2553, 8Mhz DCO and SMCLK /2

HEX
\ 0n - set tone pitch n
\ output a square wave at P1.5 (pin 7)      mk0

: P15SEC  020 dup p1 1+ cset  026 cset ;  \ set P1.5 using TA0.0   

: P15IO   020 026 cclr  020 041 cclr ;    \ set p1.5 as GPIO

: TON-  zero 0160 ! p15io ;  \ stop the timer outputting a square wave

: TON+ \ n --                \ start timer-A with interval n
  p15sec     \ init pin
  0080  0162 !  \ CCTL0  set timer output mode
  ( n ) 0172 !  \ CCR0   set interval
  0254  0160 !  \ CTL    start timer clock, mode and divider
  ;

\ --> intermediate-stage-0.txt     
\ up to here can be programmed using the intermediate-stage-0.txt file


DECIMAL      \ MIDI Tones defined B3 to B5, 2 Octaves; pitch list.   mk0
: 00  ton- ;
: 01  7962 ton+ ; \ B3 
: 02  7515 ton+ ; \ C4 
: 03  6695 ton+ ; \ D4 
: 04  5965 ton+ ; \ E4 
: 05  5630 ton+ ; \ F4 
: 06  5016 ton+ ; \ G4 
: 07  4469 ton+ ; \ A4 
: 08  3981 ton+ ; \ B4 
: 09  3758 ton+ ; \ C5 
: 0A  3348 ton+ ; \ D5 
: 0B  2983 ton+ ; \ E5 
: 0C  2815 ton+ ; \ F5 
: 0D  2508 ton+ ; \ G5 
: 0E  2235 ton+ ; \ A5 
: 0F  1991 ton+ ; \ B5 




\ The next group of Instructions covers 
\ 1n - 10 to 1F, 
\      This instruction just stores the hex number into the Output register, 
\      no other register are affected

\ 1n - Load Out with number n \   mk1
HEX 
: 10 0000 out. ; 
: 11 0001 out. ; 
: 12 0002 out. ; 
: 13 0003 out. ; 
: 14 0004 out. ; 
: 15 0005 out. ; 
: 16 0006 out. ; 
: 17 0007 out. ; 
: 18 0008 out. ; 
: 19 0009 out. ; 
: 1A 000A out. ; 
: 1B 000B out. ; 
: 1C 000C out. ; 
: 1D 000D out. ; 
: 1E 000E out. ; 
: 1F 000F out. ; 



\ 2n - Load the Delay Register with a number from 0 - F             mk1
\ and start the delay.

DECIMAL 
: DLY ( n -- ) PDSI 2! ; 
: SEC ( n -- ) 0000 DO 1000 ms LOOP ; 

: 20 0000 dly     1ms ; 
: 21 0001 dly 0002 ms ; 
: 22 0002 dly 0005 ms ; 
: 23 0003 dly 0010 ms ; 
: 24 0004 dly 0020 ms ; 
: 25 0005 dly 0050 ms ; 
: 26 0006 dly 0100 ms ; 
: 27 0007 dly 0200 ms ; 
: 28 0008 dly 0500 ms ; 
: 29 0009 dly 0001 sec ; 
: 2A 0010 dly 0002 sec ; 
: 2B 0011 dly 0005 sec ; 
: 2C 0012 dly 0010 sec ; 
: 2D 0013 dly 0020 sec ; 
: 2E 0014 dly 0030 sec ; 
: 2F 0015 dly 0060 sec ; 



\ 3n -  Instruction - to Jump back 0 to F locations of the PC = PC-n \   mk1

HEX 
: JB \ n --   \ subtract n from program counter 
  invert SGPC +! ; 

: 30 0000 jb ; 
: 31 0001 jb ; 
: 32 0002 jb ; 
: 33 0003 jb ; 
: 34 0004 jb ; 
: 35 0005 jb ; 
: 36 0006 jb ; 
: 37 0007 jb ; 
: 38 0008 jb ; 
: 39 0009 jb ; 
: 3A 000A jb ; 
: 3B 000B jb ; 
: 3C 000C jb ; 
: 3D 000D jb ; 
: 3E 000E jb ; 
: 3F 000F jb ; 


\ 4n -  Instruction to load register A with the number n \   mk1
HEX
: NTOA ( n -- ) abcd 3! ; \ A<=N

: 40 0000 NTOA ; 
: 41 0001 NTOA ; 
: 42 0002 NTOA ; 
: 43 0003 NTOA ; 
: 44 0004 NTOA ; 
: 45 0005 NTOA ; 
: 46 0006 NTOA ; 
: 47 0007 NTOA ; 
: 48 0008 NTOA ; 
: 49 0009 NTOA ; 
: 4A 000A NTOA ; 
: 4B 000B NTOA ; 
: 4C 000C NTOA ; 
: 4D 000D NTOA ; 
: 4E 000E NTOA ; 
: 4F 000F NTOA ; 


\ 5n -  Instruction, to Transfer register A to other locations,         mk0
\ nibble or bit including starting the PWM activity. 

HEX
: P16IO 
  0040 0022 cset    \ P1DIR   P1.6 out 
  0040 0026 cclr    \ P1SEL   P1.6 GPIO 
  ; 

: P16SEL
  0040 0022 cset    \ P1DIR   P1.6 out 
  0040 0026 cset    \ P1SEL   P1.6 select second function 
  ; 

: PWM-   
  zero 160 !  \ TA0CTL   stop timer 
  p16io 0040 0021 cclr ; \ set p1.6 I/O and clear p1.6

: PWM+  \ n --  \ init and start PWM at P1.6  
  01F4 0172 !   \ TA0CCR0    set period 16KHz at 8MHZ DCO
  00E0 0164 !   \ TA0CCTL1   set output mode 
       0174 !   \ TA0CCR1    set pulsewidth 
  0210 0160 !   \ TA0CTL     set timer mode and run 
  ;
 
: PWM!  \ n --   \ set PWM at P1.6 with register A
  dup 0= IF drop pwm- 
  ELSE 001F * pwm+ p16sel THEN ;

: 50 ; \ not defined instruction, NOP reaction 
: 51 abcd 3@ abcd 2! ;                             \ B<=A 
: 52 abcd 3@ abcd 1! ;                             \ C<=A 
: 53 abcd 3@ abcd 0! ;                             \ D<=A 
: 54 abcd 3@ 12wo 0! ;                             \ OUT<=A 
: 55 abcd 3@ 001 and  out> or 12wo 0! ;            \ OUT.0<=A.0
: 56 abcd 3@ 001 and  001 lshift out> or 12wo 0! ; \ OUT.1<=A.0
: 57 abcd 3@ 001 and  002 lshift out> or 12wo 0! ; \ OUT.2<=A.0
: 58 abcd 3@ 001 and  003 lshift out> or 12wo 0! ; \ OUT.4<=A.0
: 59 abcd 3@ dup 12wo 1! pwm! ;                    \ PWM<=A   mk0
: 5A ; \ not defined instruction, so the reaction is NOP - no operation
: 5B ; \ not defined instruction, NOP, the program counter 
: 5C ; \ not defined instruction,      is just incremented
: 5D ; \ not defined instruction,      nor affecting anything
: 5E ; \ not defined instruction,      can be used to replace
: 5F ; \ not defined instruction,      any Debug Instructions added in


\  We just emulate the original TPS version, 128 bytes, 
\  as we have run out of Flash memory space to extend it,
\  but later on, we might double program memory size from 128 to 256 
\  on another processor or others might just want to do it 
\  as programming exercise.



\ 6n -  Instruction, Transfers nibbles from other locations to register A   mk0

BIN
: GETIN  \ move digital input port pin values to IN register nibbles
  in@ PDSI 0! ; 

: 60 ; \ not defined instruction, NOP reaction 
: 61 abcd 2@ abcd 3! ; \ A<=B 
: 62 abcd 1@ abcd 3! ; \ A<=C 
: 63 abcd 0@ abcd 3! ; \ A<=D 
: 64 getin pdsi 0@ abcd 3! ; \ A<=IN 
: 65 getin pdsi 0@ 0001 and abcd 3@ or abcd 3! ; \ A.0<=IN.0
: 66 getin pdsi 0@ 0010 and abcd 3@ or abcd 3! ; \ A.0<=IN.1
: 67 getin pdsi 0@ 0100 and abcd 3@ or abcd 3! ; \ A.0<=IN.2
: 68 getin pdsi 0@ 1000 and abcd 3@ or abcd 3! ; \ A.0<=IN.3
: 69 ; \ not defined instruction, NOP reaction
: 6A ; \ not defined instruction, NOP reaction 
: 6B ; \ not defined instruction, NOP reaction
: 6C ; \ not defined instruction, NOP reaction
: 6D ; \ not defined instruction, NOP reaction
: 6E ; \ not defined instruction, NOP reaction
: 6F ; \ not defined instruction, NOP reaction 



\ 7n -  Instruction - maths and logic \   mk0

HEX

: 70 ; \ not defined instruction, NOP reaction
: 71 abcd 3@ 1+ abcd 3! ;          \ A<=A+1 
: 72 abcd 3@ 1- abcd 3! ;          \ A<=A-1 
: 73 abcd 3@ abcd 2@ + abcd 3! ;   \ A<=A+B 
: 74 abcd 3@ abcd 2@ - abcd 3! ;   \ A<=A-B 
: 75 abcd 3@ abcd 2@ * abcd 3! ;   \ A<=A*B 
: 76 abcd 3@ abcd 2@ / abcd 3! ;   \ A<=A/B 
: 77 abcd 3@ abcd 2@ and abcd 3! ; \ A=(A AND B)
: 78 abcd 3@ abcd 2@ or abcd 3! ;  \ A<=(A OR B)
: 79 abcd 3@ abcd 2@ xor abcd 3! ; \ A<=(A XOR B)
: 7A abcd 3@ invert abcd 3! ;      \ A<=notA 
: 7B ; \ not defined instruction, NOP reaction
: 7C ; \ not defined instruction, NOP reaction
: 7D ; \ not defined instruction, NOP reaction
: 7E ; \ not defined instruction, NOP reaction
: 7F ; \ not defined instruction, NOP reaction



\ 8n -  Instruction - Set the Page Register \   mk1
\                     This is used to jump more then 16 locations
HEX 
: LDPGE \ n --   \ store n as page preset.  
  PDSI 3! ; 

: 80 0000 ldpge ; 
: 81 0001 ldpge ; 
: 82 0002 ldpge ; 
: 83 0003 ldpge ; 
: 84 0004 ldpge ; 
: 85 0005 ldpge ; 
: 86 0006 ldpge ; 
: 87 0007 ldpge ; 
: 88 ; \ not defined instruction, NOP reaction
: 89 ; \ not defined instruction, NOP reaction
: 8A ; \ not defined instruction, NOP reaction
: 8B ; \ not defined instruction, NOP reaction
: 8C ; \ not defined instruction, NOP reaction
: 8D ; \ not defined instruction, NOP reaction
: 8E ; \ not defined instruction, NOP reaction
: 8F ; \ not defined instruction, NOP reaction
\ This will be extended to work on 256 bytes later, double the original


\ 9n -  Instruction - Jump to any location,                       mk1
\ the page register has to be set first. 

HEX
: JMP      \ n --   \ Jump to an address using preset page addr.
  1-       \          do n-1 because counter is incremented in nexi afterwards.
  SGPC 0!  \          set program counter C nibble to n.
  PDSI 3@ 008 or  PDSI 3!  ;  \ set the page preset flag.

: 90 0000 jmp ; 
: 91 0001 jmp ; 
: 92 0002 jmp ; 
: 93 0003 jmp ; 
: 94 0004 jmp ; 
: 95 0005 jmp ; 
: 96 0006 jmp ; 
: 97 0007 jmp ; 
: 98 0008 jmp ; 
: 99 0009 jmp ; 
: 9A 000A jmp ; 
: 9B 000B jmp ; 
: 9C 000C jmp ; 
: 9D 000D jmp ; 
: 9E 000E jmp ; 
: 9F 000F jmp ; 



\ An -  Instruction -  Multiply register with the number n   C * n \   mk0

HEX
: C* ( n -- ) abcd 1@ * abcd 1! ; 

: A0 0000 c* ; 
: A1 0001 c* ; 
: A2 0002 c* ; 
: A3 0003 c* ; 
: A4 0004 c* ; 
: A5 0005 c* ; 
: A6 0006 c* ; 
: A7 0007 c* ; 
: A8 0008 c* ; 
: A9 0009 c* ; 
: AA 000A c* ; 
: AB 000B c* ; 
: AC 000C c* ; 
: AD 000D c* ; 
: AE 000E c* ; 
: AF 000F c* ; 



\ Bn -  Instruction - Multiply register D with value n    D * n \   mk0

HEX
: D* ( n -- ) abcd 0@ * abcd 0! ; 

: B0 0000 d* ; 
: B1 0001 d* ; 
: B2 0002 d* ; 
: B3 0003 d* ; 
: B4 0004 d* ; 
: B5 0005 d* ; 
: B6 0006 d* ; 
: B7 0007 d* ; 
: B8 0008 d* ; 
: B9 0009 d* ; 
: BA 000A d* ; 
: BB 000B d* ; 
: BC 000C d* ; 
: BD 000D d* ; 
: BE 000E d* ; 
: BF 000F d* ; 





\ Cn -  Instructions - To take decisions and continue in 2 directions
\       The comparison either continues to the next instruction 
\       Or skips over the next instruction    \ mk1

HEX
: NEXI        \ --     \ proceed to next instruction 
  1 sgpc +!            \ increment address
  pdsi 3@ 007 > IF 
    pdsi 3@ 007 and   dup sgpc 1! pdsi 3! 
  THEN ;
: SKIP  \ f --         \ if f is true, skip.
  IF NEXI THEN ; 

: C0 ; 
: C1 abcd 3@ abcd 2@ > skip ;       \ A>B 
: C2 abcd 3@ abcd 2@ < skip ;       \ A<B 
: C3 abcd 3@ abcd 2@ = skip ;       \ A=B 

BIN
: C4 pdsi 0@ 0001 and 0001 = skip ; \ IN.0=1
: C5 pdsi 0@ 0010 and 0010 = skip ; \ IN.1=1
: C6 pdsi 0@ 0100 and 0100 = skip ; \ IN.2=1
: C7 pdsi 0@ 1000 and 1000 = skip ; \ IN.3=1

: C8 pdsi 0@ 0001 and 0= skip ;     \ IN.0=0
: C9 pdsi 0@ 0010 and 0= skip ;     \ IN.1=0
: CA pdsi 0@ 0100 and 0= skip ;     \ IN.2=0
: CB pdsi 0@ 1000 and 0= skip ;     \ IN.3=0
HEX
: CC s1? invert skip s1- ;          \ S1=0
: CD s2? invert skip s2- ;          \ S2=0

: CE s1? skip s1- ;                 \ S1=1
: CF s2? skip s2- ;                 \ S2=1



\ Dn - Instruction - Call a Subroutine                               mk0
\      - only one allowed, no nesting yet 

HEX
: CALL \ n --    \ Save the return address, 
                 \ then call subroutine n in preset page.
  NEXI           \ current program counter + 1 is return address,
  SGPC @ RTRN !  \ save it.
  jmp  ;         \ jump to n in preset page.

: D0 0000 call ; 
: D1 0001 call ; 
: D2 0002 call ; 
: D3 0003 call ; 
: D4 0004 call ; 
: D5 0005 call ; 
: D6 0006 call ; 
: D7 0007 call ; 
: D8 0008 call ; 
: D9 0009 call ; 
: DA 000A call ; 
: DB 000B call ; 
: DC 000C call ; 
: DD 000D call ; 
: DE 000E call ; 
: DF 000F call ; 



\ En -  Instruction - Return from Subroutine \   mk0
: E0  \ --                         \ copy RETN address to SGPC.
  rtrn @ sgpc !  sgpc 1@ pdsi 3! ;   
  
\        E1..EF are not assigned yet


\ Fn - Instruction - using F0..FF,  are not assigned yet
\ Just used for reloading the initial EEPROM Code 
\ when FF FF found at address 0 and 1

\ --> intermediate-stage-1.txt



\ -----------------------------------------------------------
\ Support to RUN and DEBUG MyCo programs

\ RUN starting at zero page.

HEX 
: NOINDI  \ --  \ indicator LED off
  40 59 ;

: INIT   \ --   \ initialisation of the environment
  SGPC 005 cells 000 fill  \ clr registers,
  ee0 SGPC !               \ set start address of programs,
  initio hex               \ set all I/O and number base,
  ee0 @ FFFF = IF prg0 THEN   \ check for default program.
  noindi zero out. ;       \  "GUI" off
  
  
: OP@ \ -- n \                  get op code  
  SGPC @ c@ ; 

: EVAL  \ n -- \                evaluate instruction 
  <# zero # # #> evaluate ;

: EXOP  \ --   \                execute opcode
  op@ eval nexi ; 

  
: RUN  \ --   \                 run a programm. 
  init s1- s2-
  BEGIN 
  exop 
  key? s2? s1? or or UNTIL  \   enter Forth on any key
  s2? s1? or IF s1- s2- 
  ELSE key drop THEN 
  zero out. ; 

\ LOG program to do Step by Step.  
HEX
: 4#    \ n --   \                display 4 digits unsigned 
  <# zero # # # # #> TYPE SPACE ; 
: "RNM \ --  \                    type register names
  ." SGPC UUOP RTRN PDSI ABCD 12WO" ; 
: .RGS \ -- \                     dump registers 
  base @ >r hex 
  SGPC @ 4# 
  op@    4#  
  rtrn @ 4# 
  pdsi @ 4# 
  abcd @ 4# 
  12wo @ 4# 
  r> base ! ; 

: LOG   \ --   \ LOG uses the space bar to log program                  mk0
  init cr  "rnm
  BEGIN cr .rgs 
  key 020 = WHILE exop REPEAT ; \ Enter Forth on any key but blank 
                                \ ( Space Bar )


\ Note: Use a VT100 terminal emulator. For example the 4e4th IDE or Teraterm

\ Single STEP through the entire program and dump information

HEX
: 2#    \ n --   \             display 2 digits unsigned 
  <# zero # # #> TYPE SPACE ; 

: PMAP  \ --   \               print program memory map
  zero 003 at-xy   \           start position
  004 spaces 010 zero DO i 
  2# space LOOP cr  \          print horizontal ruler
  008 zero DO cr i 2#  \       print vertical ruler, 8 pages
  010 zero DO 
  ee0 i j 010 * + + c@ 
  space 2# 
  LOOP LOOP ; 

: POS   \ --   \               cursor pointing at code in program dump.  
  sgpc 0@ 004 * 004 +  
  sgpc 1@ 005 + 
  at-xy ; 

: STEP  \ --   \               use space bar 
               \               to single step through program     mk0
  init page "rnm pmap  
  BEGIN 000 001 at-xy .rgs  
  pos  key 020 = \             stop loop, enter Forth on any key but blank
  WHILE exop REPEAT  
  000 00D at-xy ;

\ Footnote: PAGE is a standard Forth Word. It is an alias for 'clear screen'.
\ Do not mix up PAGE with the MyCo program page register.

\ --> intermediate-stage-2.txt


\ -----------------------------------------------------------
\ Simulate 'EEPROM' by using INFO-C and -D flash memory. 

\ Note:
\ The idea is to have a cache in RAM, edit there, and flush it back 
\ to mass storage when done.

\ Have a cache in 4e4th - where is RAM for it?
\ An info segment uses 64 bytes.
\ The Address Range is $1000 .. $107F 
\ That means  128 bytes or 8 blocks of 16 bytes each.
\ The Forth Scratch PAD is 80 bytes long.
\ PAD is not used inside of Forth.
\ So a segment buffer, our cache, may be built in the PAD area.
\ Only whole segments of flash can be written.
\ So get the relevant segment into cache, edit the block there,
\ and then flush it back when done.


\ Naming of memory segments, cache, These are variables and constants:

HEX
: wipecache  \ -- \                fill cache with zeros, set variables...
  pad segsiz 0 fill  
  0 upd !  FFFF blk !  0 seg !  ;

: update  \ --  \                  mark the cache as updated
  FFFF upd ! ;

  
\ Examine 'eeprom' and cache 

: ?EE   \ --  \                    dump 'eeprom' to screen (testing utility)
  base @  
  hex ee0 segsiz 2* dump 
  base ! ; 

: ?CA  \ dump cache
  base @  cr blk ?  upd ?
  hex pad segsiz dump 
  base ! ;

  
  
\                                   move 'eeprom' segments to cache and back

: eeadr   \ n -- adr \              calculate address of segment
  1 and   \ n = 0..1  only
  segsiz * ee0 + ;

: cache  \ n -- \                   copy info segment from adr to cache 
  eeadr pad segsiz move ;  

: backup  \ n -- \                  copy cache back to segment   ok
  eeadr >r                 \        make addr a local variable
  r@ segsiz flerase  \              clear flash segment
  pad r> segsiz d->i ; \            move data to instruction memory (flash) 

: FLUSH  \ --  \                    if updated copy pad to segment 
  upd @ IF seg @ backup THEN ; 



\ Move block to cache

: blkadr  \ n -- adr \              calculate address of block in cache
  3 and   \ n = 0..3  only
  blksiz * pad +  ;

: BLOCK \ n -- adr \                cache block n
  dup  blk @ = IF blkadr exit THEN  \ block is in cache already.
  flush
  7 and blk !    \                    0..7 blocks only
  blk @ 3 >   1 and   seg !  \        calculate segment 
  seg @ cache    \                    get segment
  0 upd !        \                    mark as unchanged
  blk @ blkadr ; \                    put block address on stack

  

\ Edit items in a block
HEX
: dolist \ adr -- \                   print a block 
  dup blksiz + swap 
  DO i c@ 2# LOOP ;

: ruler \ -- \                        print a block ruler
  blksiz 0 DO i 2# LOOP ;

: LIST \ n --  \                      list block n to screen
  cr ruler 
  cr block dolist ; 

: >>  \ b n -- \                      poke byte and show result.
  blk @ blkadr + c!   \               write byre b into n-th position 
                      \               of current block in cache
  blk @ list update ;
  

\ prg0 ee?  \                         test fill info segments C and D.


\ --> intermediate-stage-3.txt


\ -----------------------------------------------------------
\ EDIT a Page of memory using S1 and S2 on MyCo-board, no serial interface used.
\ Edit in cache, and when done save back to Flash used as  'eeprom'.

\ Select page
\ Start at position zero in page
\ Display address
\ Edit high nibble
\ Edit low nibble
\ Auto-increment position in page
\ Flush if done
\ Select byte


HEX
\ More switch conditions 

: S1?? \ -- f   \ true if S1 is down  
  080 020 cget  0= ;
  
: S2?? \ -- f   \ true if S2 is down
  008 020 cget  0= ;
  
: S1DOWN? \ -- f  \ true if S1 is held down
  s1?? s1?? and s1?? and ;   \ debouncing 
  
: S2DOWN? \ -- f  \ true if S2 is held down
  s2?? s2?? and s2?? and ;   \ debouncing 
  
: SWAIT \ -- \ wait till switch is pressed  
  s1- s2- BEGIN s2? s1? or UNTIL  
  29 ;  \ 1s delay - time to press the other switch too. 
  \ this works because pressing a switch is hardware detected 
  \ by it's faling edge


\ More on indicator LED "GUI"  (PWM driven LED)

: BRIGHT \ --  \ bright green LED indicating upper nibble
  4F 59 ;
  
: DIMMED \ --  \ dimmed green LED indicating lower nibble
  41 59 ;
  
: SHADED \ --  \ shaded green LED indicating lower nibble
  44 59 ;
  
: setpwm \ n -- \ set pwm to n  (n=0..F)
  0040 + eval 59   23 ( delay ) ;
  
: SWELL  \ -- 
  0010 0006 DO i setpwm     LOOP
  0001 000F DO i setpwm -1 +LOOP 
  0006 0000 DO i setpwm     LOOP 
  ;

 \ Edit both nibbles in a byte
 
: INC-OUT \  increment OUT
  out> 1+ out.  ;
  
: EDOUT \ n -- n' \ edit nibble n using S1 to inc and S2 to exit
  s1- s2- out. 
  BEGIN  \ single step or hold down S1 to increment n
  s1down? 
  IF inc-out   27 ( delay ) THEN  
  s2? UNTIL 
  s1- s2-  \ clear flags 
  out> ;   \ put n' on stack
  
: EDHI   \ adr -- \ edit nibble-1 in variable at address
  bright 
  >r  
  r@ 1@ edout r> 1! ;
  
: EDLO   \ adr -- \ edit nibble-0 in variable at address
  dimmed  
  >r  
  r@ 0@ edout r> 0! ; 
  
: EDC \ adr --  \ edit byte at address of cache 
  sgpc !            \ save addr to cache pointer
  sgpc @ c@ abcd c! \ get byte to abcd
  noindi 28         \ delay 
  abcd edhi         \ set new value in abcd nibble c
  noindi 28 
  abcd edlo         \ set new value in abcd nibble d
  sgpc @ c@  abcd c@ <>   \ if altered ...
  IF abcd c@  sgpc @ c! update THEN ;  \ store and set update flag

: EDBYTE \ -- \ edit byte at current position in current page 
  blk @ blkadr rtrn 0@ +   edc ;

: SELPAGE \ --  \ select page using S1 to inc and S2 to exit
  swell swell swell
  zero edout block drop  \ make SELPAGE = BLOCK in forth
  noindi ;

: SELPOS \ --  \ select position in page
  rtrn 0@ edout rtrn 0! ;  \ we use variable RTRN as pointer to byte in page

: EDPAGE \ -- \ edit current page ( = block number )
  zero rtrn !  \ start with byte zero
  BEGIN 
  swell swell  rtrn 0@ out.  
  swait s1? s2? and 0= WHILE
  s2down? IF swell selpos THEN  \ select position in page if S2 is held down
  edbyte                        \ edit byte at selected position
  rtrn 0@ 1+ rtrn 0!            \ increment position
  REPEAT
  flush noindi zero out. 
  s1- s2- ;

: SEDIT \ --   \ edit MyCo program page using S2 and S1
  selpage edpage ;
  
\ --> intermediate-stage-4.txt

\ -----------------------------------------------------------
\ MyCo 
\ User interface of MyCo: 
\ Buttons S1 and S2, 4 LEDs, Reset Button.

HEX 
: BLINK  \  blink 4 LEDs to indicate wait state  mk
  000f out. 28  \ 500 ms delay   
  zero out. 28  ;

: MAIN 
  blink 
  s1? IF run   exit THEN   \ press S1 to run program
  s2? IF sedit exit THEN   \ press S2 to enter S1S2 edit mode
  ; 

: Q?  \ -- f   \ quit - press q to leave MyCo 
  key? if key 0071 ( q ) = else zero then ; 

: MYCO   
  28                   \ 500 ms delay    
  ee0 @ IF prg0 THEN   \ check, load default program  
  init s1- s2-  noindi \ initialisation
  BEGIN main Q? UNTIL ." Forth "  ;  \ mk0 

\ --> intermediate-stage-5.txt

\ ' myco app !
\ save 

\ decimal mem u. 


\ todo ***
\ verify opcodes 



( finis )
